@@ -2,11 +2,11 @@
 
 push @ARGV, "";
 
-if($ARGV[0] eq "--help")
+if ($ARGV[0] eq "--help")
 {
 print STDERR <<EOT;
-usage:	bdftopsf [+h][+u|-u][+1|-1]|[-h[+r|-r]] [+o|-o OUTPUT]
-	[--] [INPUT [TABLE...]]
+usage:	bdftopsf [[+h] [+u|-u] [+1|-1]]|[-h [+r|-r]]
+	[+o|-o NAME] [--] [INPUT [TABLE...]]
 
 +h	PSF mode - write a PSF header. This is the default.
   +u	Write an unicode table. Default for fonts with more
@@ -20,7 +20,8 @@ usage:	bdftopsf [+h][+u|-u][+1|-1]|[-h[+r|-r]] [+o|-o OUTPUT]
 -h	RAW mode - write no PSF header (and no unicode table,
 	of course).
   +r	Reject 512 character fonts. This is the default. Most
-	(if not all) UNIX systems won't handle these properly.
+	systems won't handle these properly without some sort
+	of header.
   -r	Allow 512 character fonts.
 
 +o	Output to INPUT.psf or INPUT.raw (replacing the .bdf in
@@ -30,17 +31,17 @@ usage:	bdftopsf [+h][+u|-u][+1|-1]|[-h[+r|-r]] [+o|-o OUTPUT]
 --	Terminate the option list.
 
 INPUT	RAW and PSF1: a 256 or 512 character BDF file with
-	width = 8 and height between 1 and 255. PSF2: a BDF
-	with width between 1 and 1200, height between 1 and
-	2400 and number of characters between 1 and 2048.
+	width = 8 and height between 1 and 255; PSF2: a BDF
+	file with width between 1 and 128, height between 1 and
+	256 and number of characters between 1 and 65560. If no
+	input is specified, the standard input is used.
 
 TABLE	A duplicate unicodes table. Each line must either be
 	blank or contain two hexadecimal unicodes, each
 	consisting of maximum 4 digits. Used for PSF with
-	unicode data only, otherwise treated as error.
+	unicode table only, otherwise treated as error.
 
-If no input or output are specified, the standard output is
-the standard input and output are used.
+If no output is specified, the standard output is used.
 
 Any options not specified in the above order are treated as
 non-option arguments.
@@ -48,10 +49,10 @@ EOT
 	exit 0;
 }
 
-if($ARGV[0] eq "--version")
+if ($ARGV[0] eq "--version")
 {
 print STDERR <<EOT;
-bdftopsf.pl 0.2.0, Copyright (C) 2005 Dimitar Toshkov Zhekov
+bdftopsf.pl 0.2.1, Copyright (C) 2008 Dimitar Toshkov Zhekov
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -68,7 +69,7 @@ EOT
 	exit 0;
 }
 
-if($ARGV[0] eq "-h")
+if ($ARGV[0] eq "-h")
 {
 	$header = 0;
 	$suffix = ".raw";
@@ -76,7 +77,7 @@ if($ARGV[0] eq "-h")
 	$version = 0;
 	shift @ARGV;
 
-	if($ARGV[0] eq "-r")
+	if ($ARGV[0] eq "-r")
 	{
 		$reject = 0;
 		shift @ARGV;
@@ -84,7 +85,7 @@ if($ARGV[0] eq "-h")
 	else
 	{
 		$reject = 1;
-		if($ARGV[0] eq "+r") { shift @ARGV; }
+		if ($ARGV[0] eq "+r") { shift @ARGV; }
 	}
 }
 else
@@ -92,81 +93,87 @@ else
 	$header = 1;
 	$suffix = ".psf";
 	$reject = 0;
-	if($ARGV[0] eq "+h") { shift @ARGV; }
+	if ($ARGV[0] eq "+h") { shift @ARGV; }
 
-	if($ARGV[0] eq "+u")
+	if ($ARGV[0] eq "+u")
 	{
 		$unicode = 1;
 		shift @ARGV;
 	}
-	elsif($ARGV[0] eq "-u")
+	elsif ($ARGV[0] eq "-u")
 	{
 		$unicode = 0;
 		shift @ARGV;
 	}
 
-	if($ARGV[0] eq "+1")
+	if ($ARGV[0] eq "+1")
 	{
 		$version = 0;
 		shift @ARGV;
 	}
-	elsif($ARGV[0] eq "-1")
+	elsif ($ARGV[0] eq "-1")
 	{
 		$version = 1;
 		shift @ARGV;
 	}
 }
 
-if($ARGV[0] eq "+o")
+if ($ARGV[0] eq "+o")
 {
 	$output = "";
 	shift @ARGV;
 }
-elsif($ARGV[0] eq "-o")
+elsif ($ARGV[0] eq "-o")
 {
 	shift @ARGV;
-
-	$ARGV[0] ne "" || die("$0: -o requires a non-empty argument\n");
+	$#ARGV > 0 || die("$0: -o requires an argument\n");
 	$output = $ARGV[0];
 	shift @ARGV;
 }
+elsif ($ARGV[0] =~ /^-o(.*)$/)
+{
+	$output = $1;
+	shift @ARGV;
+}
 
-if($ARGV[0] eq "--") { shift @ARGV; }
-elsif($ARGV[0] =~ /^([-+][0-9a-z])$/) { print STDERR "$0: suspicuous $1, use -- to terminate the option list\n"; }
+if ($ARGV[0] eq "--") { shift @ARGV; }
+elsif ($ARGV[0] =~ /^([-+][0-9a-z])$/) { print STDERR "$0: suspicuous $1, use -- to terminate the option list\n"; }
 pop @ARGV;
 
-if($#ARGV < 0) { $ARGV[0] = "-"; }
+if ($#ARGV < 0) { $ARGV[0] = "-"; }
 
 open(BDF, "<$ARGV[0]") || die("$0: $ARGV[0]: $!\n");
 
-while(<BDF>)
+while (<BDF>)
 {
-	if(/^FONTBOUNDINGBOX\s+([0-9]+)\s+([0-9]+).*$/)
+	if (/^FONTBOUNDINGBOX\s+([0-9]+)\s+([0-9]+).*$/)
 	{
 		$width = $1;
 		$height = $2;
 	}
-	elsif(/^CHARS\s+([0-9]+)$/)
+	elsif (/^CHARS\s+([0-9]+)$/)
 	{
 		$chars = $1;
 		last;
 	}
 }
 
-($width > 0 && $width <= 1200) || die("$0: $ARGV[0]: width $width zero or too big\n");
-($height > 0 && $height <= 2400) || die("$0: $ARGV[0]: height $height zero or too big\n");
-($chars > 0 && $chars <= 2048) || die("$0: $ARGV[0]: CHARS $chars zero or too big\n");
+(defined($width) && defined($chars)) || die("$0: $ARGV[0]: missing width/height or CHARS\n");
+# psf2 can handle almost anything, but there must be some reasonable limits
+($width > 0 && $width <= 128) || die("$0: $ARGV[0]: width $width out of range\n");
+($height > 0 && $height <= 256) || die("$0: $ARGV[0]: height $height out of range\n");
+($chars > 0 && $chars <= 65560) || die("$0: $ARGV[0]: CHARS $chars out of range\n");
 
 $minimal = $width != 8 || $height >= 256 || ($chars != 256 && $chars != 512);
 $header != 0 || $minimal == 0 || die("$0: $ARGV[0]: RAW format: invalid width $width, height $height or CHARS $chars\n");
 $chars == 256 || $reject == 0 || die("$0: $ARGV[0]: RAW format: CHARS $chars rejected, use -r to accept\n", $chars);
 
-if(!defined($unicode)) { $unicode = $chars > 256; }
-if(!defined($version)) { $version = $minimal; }
+if (!defined($unicode)) { $unicode = $chars > 256; }
+if (!defined($version)) { $version = $minimal; }
 else { $version >= $minimal || die("$0: $ARGV[0]: requested version ", $version + 1,  ", required ", $minimal + 1, "\n"); }
 
-if(!defined($output)) { $output = "-"; }
-elsif($output eq "") { if($ARGV[0] =~ /^(.*).bdf$/) { $output = "$1$suffix" ; } else { $output = "$ARGV[0]$suffix"; } }
+if (!defined($output)) { $output = "-"; }
+elsif ($output eq "") { if ($ARGV[0] =~ /^(.*).bdf$/) { $output = "$1$suffix" ; } else { $output = "$ARGV[0]$suffix"; } }
 
 $linesize = ($width + 7) >> 3;
 $charsize = $linesize * $height;
@@ -183,9 +190,9 @@ open(OUT, ">$output") || die("$0: $output: $!\n");
 $SIG{INT} = 'int';
 binmode(OUT) || bye("$0: $output: $!\n");
 
-if($header != 0)
+if ($header != 0)
 {
-	if($version == 0) { printf OUT "%c%c%c%c", 0x36, 0x04, ($chars == 512) + $unicode * 2, $height; }
+	if ($version == 0) { printf OUT "%c%c%c%c", 0x36, 0x04, ($chars == 512) + $unicode * 2, $height; }
 	else
 	{
 		printf OUT "%c%c%c%c", 0x72, 0xB5, 0x4A, 0x86;
@@ -200,53 +207,53 @@ if($header != 0)
 }
 
 $lines = 0;
-while(<BDF>)
+while (<BDF>)
 {
 	$bytes = 0;
-	while(/^([0-9a-fA-F]{2})(([0-9a-fA-F]{2})*)$/)
+	while (/^([0-9a-fA-F]{2})(([0-9a-fA-F]{2})*)$/)
 	{
 		printf OUT "%c", hex($1);
 		$bytes++;
 		$_ = $2;
 	}
-	if($bytes != 0)
+	if ($bytes != 0)
 	{
 		$lines++;
 		$bytes == $linesize || bye("$0: $ARGV[0]: invalid number of bytes $bytes on data line $lines\n");
 	}
 	else
 	{
-		if(/^ENCODING\s+([0-9]+)$/)
+		if (/^ENCODING\s+([0-9]+)$/)
 		{
 			push @unimap, $1;
-			if($1 != 65535) { $unidup{$1}[0] = $1; }
+			if ($1 != 65535) { $unidup{$1}[0] = $1; }
 		}
-		elsif(/^ENDFONT$/) { last; }
+		elsif (/^ENDFONT$/) { last; }
 	}
 }
 
 close BDF;
 
-if($#ARGV > 0)
+if ($#ARGV > 0)
 {
-	if($unicode)
+	if ($unicode)
 	{
 		do
 		{
 			shift @ARGV;
 
 			open(DUP, "<$ARGV[0]") || bye("$0: $ARGV[0]: $!\n");
-			while(<DUP>)
+			while (<DUP>)
 			{
 				next if /^\s*$/;
 				/^([0-9a-fA-F]{1,4})\s+([0-9a-fA-F]{1,4})$/ || bye("$0: $ARGV[0]: invalid unicode(s) $_\n");
 				$duplicate = hex($2);
 				hex($1) != $duplicate || bye("$0: $ARGV[0]: invalid duplicate entry $_\n");
-				foreach(@{$unidup{hex($1)}}) { if($_ == $duplicate) { $duplicate = 65535; last; } }
-				if($duplicate != 65535) { push @{$unidup{hex($1)}}, $duplicate; }
+				foreach (@{$unidup{hex($1)}}) { if ($_ == $duplicate) { $duplicate = 65535; last; } }
+				if ($duplicate != 65535) { push @{$unidup{hex($1)}}, $duplicate; }
 			}
 			close DUP;
-		} while($#ARGV > 0);
+		} while ($#ARGV > 0);
 	}
 	else { bye("$0: invalid number of arguments\n"); }
 }
@@ -254,21 +261,22 @@ if($#ARGV > 0)
 @unimap == $chars || bye("$0: $output: invalid number of chars @unimap\n");
 @unimap * $charsize == $lines * $linesize || bye("$0: $output: invalid number of data lines $lines\n");
 
-if($unicode)
+if ($unicode)
 {
-	foreach(@unimap)
+	foreach (@unimap)
 	{
-		foreach(@{$unidup{$_}}) {
-			if($version == 0) { printf OUT "%c%c", $_ & 0xFF, $_ >> 8; }
-			elsif($_ <= 0x7F) { printf OUT "%c", $_; }
-			else {
-				if($_ <= 0x7FF) { printf OUT "%c", 0xC0 + ($_ >> 6); }
+		foreach (@{$unidup{$_}}) {
+			if ($version == 0) { printf OUT "%c%c", $_ & 0xFF, $_ >> 8; }
+			elsif ($_ <= 0x7F) { printf OUT "%c", $_; }
+			else
+			{
+				if ($_ <= 0x7FF) { printf OUT "%c", 0xC0 + ($_ >> 6); }
 				else { printf OUT "%c%c", 0xE0 + ($_ >> 12), 0x80 + (($_ >> 6) & 0x3F); }
 				printf OUT "%c", 0x80 + ($_ & 0x3F);
 			}
 		}
 		printf OUT "%c", 0xFF;
-		if($version == 0) { printf OUT "%c", 0xFF; }
+		if ($version == 0) { printf OUT "%c", 0xFF; }
 	}
 }
 
